from collections import defaultdict
from contextlib import contextmanager
from eventhorizon.signal import Signal


class Registry(defaultdict):
    """
    Creates a signal registry. All registries are
    isolated from one another, i.e. signals registered
    on one registry will not be visible in another
    registry.

    :param signal_class: A signal class to use,
        defaults to the regular
        :class:``eventhorizon.signal.Signal``.
    """
    def __init__(self, signal_class=Signal):
        super(Registry, self).__init__(signal_class)

    signal = defaultdict.__getitem__
    destroy = defaultdict.pop

    @contextmanager
    def emitting(self, exit, enter=None):
        """
        Emit a signal at block exit, and optionally at
        block entry, regardless if the block succeeded
        or not.

        :param exit: Signal to trigger on block exit.
        :param enter: Optional signal to trigger on
            block entry.
        """
        if enter:
            self.signal(enter).trigger()
        try:
            yield
        finally:
            self.signal(exit).trigger()
